@@ -9,7 +9,8 @@ import html
 import collections
 import functools
 import dataclasses
-from typing import Deque, MutableSequence, Optional, cast
+from typing import Optional, cast
+from collections.abc import MutableSequence
 
 from qutebrowser.qt.core import (pyqtSlot, pyqtSignal, Qt, QTimer, QDir, QModelIndex,
                           QItemSelectionModel, QObject, QEventLoop, QUrl)
@@ -89,7 +90,7 @@ class PromptQueue(QObject):
         self._question = None
         self._shutting_down = False
         self._loops: MutableSequence[qtutils.EventLoop] = []
-        self._queue: Deque[usertypes.Question] = collections.deque()
+        self._queue: collections.deque[usertypes.Question] = collections.deque()
         message.global_bridge.mode_left.connect(self._on_mode_left)
 
     def __repr__(self):
@@ -341,9 +342,9 @@ class PromptContainer(QWidget):
         """Leave KEY_MODE whenever a prompt is aborted."""
         try:
             modeman.leave(self._win_id, key_mode, 'aborted', maybe=True)
-        except objreg.RegistryUnavailableError:
+        except (objreg.RegistryUnavailableError, RuntimeError):
             # window was deleted: ignore
-            pass
+            log.prompt.debug(f"Ignoring leaving {key_mode} as window was deleted")
 
     @pyqtSlot(usertypes.KeyMode)
     def _on_prompt_done(self, key_mode):
@@ -653,6 +654,12 @@ class FilenamePrompt(_BasePrompt):
 
     """A prompt for a filename."""
 
+    # Note: This *must* be a class variable! If it's not, for unknown reasons,
+    # we get a segfault in Qt/PyQt in QFileInfoGatherer::getInfo() if we have
+    # nested download prompts (i.e. trigger a download while a download prompt
+    # is open already).
+    _null_icon_provider = NullIconProvider()
+
     def __init__(self, question, parent=None):
         super().__init__(question, parent)
         self._init_texts(question)
@@ -752,7 +759,7 @@ class FilenamePrompt(_BasePrompt):
         self._file_model = QFileSystemModel(self)
 
         # avoid icon and mime type lookups, they are slow in Qt6
-        self._file_model.setIconProvider(NullIconProvider())
+        self._file_model.setIconProvider(self._null_icon_provider)
 
         self._file_view.setModel(self._file_model)
         self._file_view.clicked.connect(self._insert_path)
@@ -976,12 +983,22 @@ class YesNoPrompt(_BasePrompt):
             raise Error("Invalid value {} - expected yes/no!".format(value))
 
         if save:
+            value = self.question.answer
             opt = config.instance.get_opt(self.question.option)
-            assert isinstance(opt.typ, configtypes.Bool)
+            if isinstance(opt.typ, configtypes.Bool):
+                pass
+            elif isinstance(opt.typ, configtypes.AsBool):
+                value = opt.typ.from_bool(value)
+            else:
+                raise AssertionError(
+                    f"Cannot save prompt answer ({opt.name}). Expected 'Bool' or 'AsBool' "
+                    f"type option, got: value={value} type={type(opt.typ)}"
+                )
+
             pattern = urlmatch.UrlPattern(self.question.url)
 
             try:
-                config.instance.set_obj(opt.name, self.question.answer,
+                config.instance.set_obj(opt.name, value,
                                         pattern=pattern, save_yaml=True)
             except configexc.Error as e:
                 raise Error(str(e))
